// lite-c newton demo (c) 2007 thomas oppl



#include <acknex.h>
#include <default.c>
#include <d3d9.h>

// newton works with meters!
// 1 meter = 32 quants
float QUANTTOMETER = 0.03125;
float METERTOQUANT = 32;

#include "newton.h"
#include "matrix.c"
#include "newton_main.c"
#include "newton_debug.c"



ENTITY* skycube =
{
  type = "skycube+6.tga";
  flags2 = SKY | CUBE | VISIBLE;
}

var currentcollisiontype = 1;

TEXT* collisiontypestrings =
{
	string = "ball", "box", "cone", "capsule", "cylinder", "chamfered cylinder", "convex hull";
}



//--------------------------------------------------------------------------------------- onforceandtorque()
// callback which applies gravity to active physics entities
//---------------------------------------------------------------------------------------
void onforceandtorque(NewtonBody* body)
{
	float mass, ixx, iyy, izz;
	NewtonBodyGetMassMatrix(body, &mass, &ixx, &iyy, &izz);
	NewtonBodySetForce(body, vectorf(0, 0, -9.8 * mass));
}

//--------------------------------------------------------------------------------------- fireentity()
// emit a physics entity from the camera view 
//---------------------------------------------------------------------------------------
void fireentity()
{
	var power = 0;
	while(mouse_left) // power depends on how long you hold the mouse button
	{
		power = clamp(power + time_step, 0, 25);
		draw_line(vector(10, 40, 0), 0, 100);
		draw_line(vector(10, 40, 0), vector(255,255,255), 100);
		draw_line(vector(10 + power * 5, 40, 0), vector(255,255,255), 100);
		wait(1);	
	}
	
	VECTOR v;
	vec_set(&v, vector(100, 0, 0));
	vec_rotate(&v, &camera->pan);
	vec_add(&v, &camera->x);
	
	switch(currentcollisiontype)
	{
		case NEWTON_SPHERE:
			you = ent_create("ball.mdl", &v, NULL);
			break;
		case NEWTON_BOX:
			you = ent_create("crate.mdl", &v, NULL);
			break;
		case NEWTON_CONE:
			you = ent_create("cone.mdl", &v, NULL);
			you->tilt = 90;
			break;
		case NEWTON_CAPSULE:
			you = ent_create("capsule.mdl", &v, NULL);
			you->pan = 90;
			break;
		case NEWTON_CYLINDER:
			you = ent_create("tire.mdl", &v, NULL);
			you->pan = 90;
			break;
		case NEWTON_CHAMFEREDCYLINDER:
			you = ent_create("chamferedcylinder.mdl", &v, NULL);
			you->pan = 90;
			break;
		case NEWTON_CONVEXHULL:
			you = ent_create("rock.mdl", &v, NULL);
			break;
	}
	
	you->pan += camera->pan;
	NewtonBody *body = newton_addentity(you, 10, currentcollisiontype, onforceandtorque); // register entity as physics entity

	vec_set(&v, vector(power, 0, 0));
	vec_rotate(&v, &camera->pan);
	NewtonBodySetVelocity(body, vectorf(&v));
}

//--------------------------------------------------------------------------------------- quit()
// gets called when the engine closes
//---------------------------------------------------------------------------------------
void quit()
{
	newton_stop();
}

//--------------------------------------------------------------------------------------- main()
// main function
//---------------------------------------------------------------------------------------
void main()
{
	vec_set(&sky_color, nullvector);
	max_particles = 50000; // for draw_line3d()
	fps_max = 120;
	video_mode = 8;
	
	wait(3);
	level_load(""); // use an empty level
	wait(1);
	
	vec_set(&sun_angle, vector(300, 30, 0));
	vec_set(&camera->x, vector(0, 0, 500)); // camera start position
	
	
	
	// add world geometry
	you = ent_create("newton_playground.mdl", nullvector, NULL);
	you->flags |= FLAG8; // flag8 -> will be added to world collision geometry
	you = ent_create("tire.mdl", vector(-600, 1200, 120), NULL);
	you->flags |= FLAG8;
	vec_scale(&you->scale_x, 30);
	you->tilt = -90;
	you = ent_create("tire.mdl", vector(-600, 2400, 120), NULL);
	you->flags |= FLAG8;
	vec_scale(&you->scale_x, 30);
	
	
	
	newton_start();
	on_close = quit;
	
	def_moveset(); // default camera movement
	on_mouse_left = fireentity;
	
	while(1)
	{
		// display current collision type
		currentcollisiontype = cycle(currentcollisiontype + mickey.z / 120, 1, 8);
		draw_text((collisiontypestrings->pstring)[currentcollisiontype - 1], 10, 10, vector(255,255,255));
		wait(1);	
	}
}