// lite-c newton demo (c) 2007 thomas oppl



//#######################################################################################
//####################################################################################### MATRIX FUNCTIONS
//#######################################################################################



#define PI (3.14159265)
#define DEG2RAD (PI / 180.0)
#define RAD2DEG (180.0 / PI)

double atan2(double, double);
API(atan2, msvcrt)



//--------------------------------------------------------------------------------------- ent_getmatrix()
// return the transformation matrix of an entity
//---------------------------------------------------------------------------------------
void ent_getmatrix(ENTITY *entity, D3DXMATRIX *mout)
{	
	D3DXMATRIX mscale;
	D3DXMatrixScaling(&mscale, entity->scale_x, entity->scale_y, entity->scale_z);
	
	D3DXMATRIX mpan;
	D3DXMatrixRotationZ(&mpan, entity->pan * DEG2RAD);
	
	D3DXMATRIX mtilt;
	D3DXMatrixRotationY(&mtilt, -entity->tilt * DEG2RAD); // -tilt?
	
	D3DXMATRIX mroll;
	D3DXMatrixRotationX(&mroll, entity->roll * DEG2RAD);
	
	D3DXMATRIX mtranslation;
	D3DXMatrixTranslation(&mtranslation, entity->x, entity->y, entity->z);
	
	D3DXMATRIX mtemp1;
	D3DXMATRIX mtemp2;

	D3DXMatrixMultiply(&mtemp1, &mscale, &mroll);
	D3DXMatrixMultiply(&mtemp2, &mtemp1, &mtilt);
	D3DXMatrixMultiply(&mtemp1, &mtemp2, &mpan);
	D3DXMatrixMultiply(mout, &mtemp1, &mtranslation);
}

//--------------------------------------------------------------------------------------- ent_getmatrix_rb()
// return the transformation matrix of an entity (without scale -> for rigid bodies)
//---------------------------------------------------------------------------------------
void ent_getmatrix_rb(ENTITY *entity, D3DXMATRIX *mout)
{
	// ignores entity scale -> not needed for rigid bodies -> collision shape size gets defined by bounding box
	
	D3DXMATRIX mpan;
	D3DXMatrixRotationZ(&mpan, entity->pan * DEG2RAD);
	
	D3DXMATRIX mtilt;
	D3DXMatrixRotationY(&mtilt, -entity->tilt * DEG2RAD); // -tilt?
	
	D3DXMATRIX mroll;
	D3DXMatrixRotationX(&mroll, entity->roll * DEG2RAD);
	
	D3DXMATRIX mtranslation;
	D3DXMatrixTranslation(&mtranslation,
		entity->x * QUANTTOMETER,
		entity->y * QUANTTOMETER,
		entity->z * QUANTTOMETER);
	
	D3DXMATRIX mtemp1;
	D3DXMATRIX mtemp2;
	
	D3DXMatrixMultiply(&mtemp1, &mroll, &mtilt);
	D3DXMatrixMultiply(&mtemp2, &mtemp1, &mpan);
	D3DXMatrixMultiply(mout, &mtemp2, &mtranslation);
}

//--------------------------------------------------------------------------------------- ent_setmatrix_rb()
// set the transformation of an entity
//--------------------------------------------------------------------------------------- 
//
//
//
// i think a 3dgs rotation matrix gets constructed like that:
//
// pan matrix (rotation about z-axis)
// | cos(p)   -sin(p)   0 |
// | sin(p)   cos(p)    0 |
// | 0        0         1 |
//
// tilt matrix (rotation about y-axis)
// | cos(t)    0   sin(t) |
// | 0         1   0      |
// | -sin(t)   0   cos(t) |
//
// roll matrix (rotation about x-axis)
// | 1   0        0       |
// | 0   cos(r)   -sin(r) |
// | 0   sin(r)   cos(r)  |
//
// roll * tilt * pan =
//
// | cos(t)*cos(p)                            cos(t)*-sin(p)                            sin(t)         |
// | -sin(r)*-sin(t)*cos(p) + cos(r)*sin(p)   -sin(r)*-sin(t)*-sin(p) + cos(r)*cos(p)   -sin(r)*cos(t) |
// | cos(r)*-sin(t)*cos(p) + sin(r)*sin(p)    cos(r)*-sin(t)*-sin(p) + sin(r)*cos(p)    cos(r)*cos(t)  |
//
// in case of gimbal lock (cos(t) = 0, sin(r) = 0, cos(r) = 1):
//
// | 0                0                 sin(t) |
// | sin(p)           cos(p)            0      |
// | -sin(t)*cos(p)   -sin(t)*-sin(p)   0      |
//
//
//
//---------------------------------------------------------------------------------------
void ent_setmatrix_rb(ENTITY *entity, float *m)
{
	float pan, tilt, roll;
	
	if(fabs(m[2]) > 0.9999) // looking straight up or down -> gimbal lock
	{
		pan = atan2(-m[4], m[5]); // -m[4]?
		tilt = 1.570796 * sign(m[2]);
		roll = 0;
	}
	else
	{
		pan = atan2(m[1], m[0]); // swap?
		tilt = asin(m[2]);
		roll = atan2(m[6], m[10]);
	}
	
	entity->pan = pan * RAD2DEG;
	entity->tilt = tilt * RAD2DEG;
	entity->roll = roll * RAD2DEG;
	
	entity->x = m[12] * METERTOQUANT;
	entity->y = m[13] * METERTOQUANT;
	entity->z = m[14] * METERTOQUANT;
}