// lite-c newton demo (c) 2007 thomas oppl



//#######################################################################################
//####################################################################################### DEBUG UTILITIES
//#######################################################################################



var newtondebugdisplaymode = 0;



//--------------------------------------------------------------------------------------- drawaxes()
// draw the local coordinate axes of an entity
//---------------------------------------------------------------------------------------
void drawaxes(ENTITY* entity)
{
	VECTOR xaxis; vec_set(&xaxis, vector(50, 0, 0));
	VECTOR yaxis; vec_set(&yaxis, vector(0, 50, 0));
	VECTOR zaxis; vec_set(&zaxis, vector(0, 0, 50));
	
	vec_rotate(&xaxis, &entity->pan);
	vec_rotate(&yaxis, &entity->pan);
	vec_rotate(&zaxis, &entity->pan);
	vec_add(&xaxis, &entity->x);
	vec_add(&yaxis, &entity->x);
	vec_add(&zaxis, &entity->x);
	
	draw_line3d(&entity->x, NULL, 100);
	draw_line3d(&entity->x, vector(0,0,255), 100); 
	draw_line3d(&xaxis, vector(0,0,255), 100);
	draw_line3d(&entity->x, NULL, 100);
	draw_line3d(&entity->x, vector(0,255,0), 100); 
	draw_line3d(&yaxis, vector(0,255,0), 100); 
	draw_line3d(&entity->x, NULL, 100);
	draw_line3d(&entity->x, vector(255,0,0), 100); 
	draw_line3d(&zaxis, vector(255,0,0), 100); 
}

//--------------------------------------------------------------------------------------- foreachpolygon()
// gets called for each polygon and draws it
//---------------------------------------------------------------------------------------
void foreachpolygon(NewtonBody *body, int vertexcount, float *facearray, int faceid)
{
	int i;
	for(i = 0; i < vertexcount; i++)
   {
		VECTOR v;
		v.x = facearray[i*3+0] * METERTOQUANT;
		v.y = facearray[i*3+1] * METERTOQUANT;
		v.z = facearray[i*3+2] * METERTOQUANT;
		
		if(i == 0)
		{
			draw_line3d(&v, NULL, 100);
			draw_line3d(&v, vector(0,255,255), 100);
		}
		else
		{
			draw_line3d(&v, vector(0,255,255), 100); 
   	}
   }
}

//--------------------------------------------------------------------------------------- foreachbody()
// gets called for each physics entity and draws its collision debug geometry
//---------------------------------------------------------------------------------------
void foreachbody(NewtonBody *body)
{
	float mass, ixx, iyy, izz;
	NewtonBodyGetMassMatrix(body, &mass, &ixx, &iyy, &izz);

	if(newtondebugdisplaymode == 1)
	{
		if(mass != 0) // not static
		{
			if(NewtonBodyGetSleepingState(body)) // not sleeping
			{
				NewtonBodyForEachPolygonDo(body, foreachpolygon);
				ENTITY *entity = (ENTITY*)NewtonBodyGetUserData(body);
				drawaxes(entity);
			}
		}
	}
}

//--------------------------------------------------------------------------------------- newton_debugdisplay()
// display collision debug geometry
//---------------------------------------------------------------------------------------
void newton_debugdisplay()
{
	while(newtondebugdisplaymode)
	{
		NewtonWorldForEachBodyDo(nworld, foreachbody);
		wait(1);
	}
}

//--------------------------------------------------------------------------------------- newton_toggledebugdisplay()
// toggle the debug display
//---------------------------------------------------------------------------------------
void newton_toggledebugdisplay()
{
	newtondebugdisplaymode = 1 - newtondebugdisplaymode;
	if(newtondebugdisplaymode == 1) newton_debugdisplay();
}

//--------------------------------------------------------------------------------------- newton_debugdisplay_startup()
// register the key for toggling the debug display
//---------------------------------------------------------------------------------------
void newton_debugdisplay_startup()
{
	on_f12 = newton_toggledebugdisplay;
}